/* liblwevent/src/liblwevent/200_eventloop.c
 *
 *  (c)2007, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/* lwevent_epoll_fd
 *  File descriptor returned by epoll_create(). Initialised in lwevent_init().
 */
static int lwevent_epoll_fd;



/* lwevent_loop_exit
 *  Flag which, when non-zero, causes lwevent_loop() to exit after calling lwevent_wait(). 
 *  Initialised to zero in lwevent_init(). May be set by any user code.
 */
volatile int lwevent_loop_exit;



/* _lw_revents[]
 *  A dynamic array of returned events. Initialised in lwevent_init().
 */
static int _lw_revents_size;
static struct epoll_event* _lw_revents;



/* _lw_revents_count, _lw_revents_overcount
 *  A count of times that epoll_wait(2) has returned in total, and a count of times it has returned
 *  with the maximum number of events. Used to dynamically scale up the size of the retrieval buffer
 *  at runtime. Initialised in lwevent_init().
 */
static int _lw_revents_count, _lw_revents_overcount;



/* lwevent_init()
 *  Initialise lwevent_epoll_fd, lwevent_loop_exit, _lw_revents[] etc.
 */
int lwevent_init(void)
{
    _lw_revents_size = 4;
    _lw_revents = malloc(sizeof(struct epoll_event) * _lw_revents_size);
    if(!_lw_revents) return -1;
    _lw_revents_count = 0;
    _lw_revents_overcount = 0;

    lwevent_loop_exit = 0;

    lwevent_epoll_fd = TEMP_FAILURE_RETRY(
        epoll_create(4 /* initial number of file descriptors */)
    );
    if(lwevent_epoll_fd == -1) {
        free(_lw_revents);
        return -1;
    }

    if(lwevent_cloexec(lwevent_epoll_fd)) {
        free(_lw_revents);
        close(lwevent_epoll_fd);
        return -1;
    }

    return 0;
}



/* lwevent_exit()
 *  Cleans up resources used by lwevent, putting the library into a state where it is ready to be
 *  re-initialised with lwevent_init().
 */
void lwevent_exit(void)
{
    close(lwevent_epoll_fd);
    free(_lw_revents);
}



/* lwevent_loop()
 *  Convenience function which just keeps calling lwevent_wait(), with no timeout, until an error
 *  occurs or lwevent_loop_exit is set.
 */
int lwevent_loop(void)
{
    while(!lwevent_loop_exit) {
        if(lwevent_wait(-1) == -1) {
            if(errno == EINTR) continue;
            return -1;
        }
    }
    return 0;
}



/* _lwevent_invalidate(), _lwevent_invalid_head
 *  Marks the object `ev' as invalid. The associated file descriptor must already have been closed,
 *  since we set it to -1. A linked list of invalid events is created.
 */
static struct lwevent* _lwevent_invalid_head;
static void _lwevent_invalid(struct lwevent* ev) __attribute__((nonnull));
static void _lwevent_invalid(struct lwevent* ev)
{
    /* skip already invalid structures */
    if(ev->fd == -1) return;

    /* mark as invalid */
    ev->fd = -1;

    /* add to linked list */
    ev->invalid_next = _lwevent_invalid_head;
    _lwevent_invalid_head = ev;
}



/* lwevent_wait()
 *  Wrapper around epoll_wait(2). For every returned event, we assume the object has an associated
 *  struct lwevent in the struct epoll_event.
 */
int lwevent_wait(int timeout_ms)
{
    int ret, i, revents;
    struct lwevent* ev;
    struct epoll_event* new_revents;

    _lwevent_invalid_head = 0;

    /* perform epoll_wait(2) */
    ret = epoll_wait(lwevent_epoll_fd, _lw_revents, _lw_revents_size, timeout_ms);
    if(ret == -1) return -1;

    /* process returned events */
    for(i = 0; i < ret; ++i) {
        ev = (struct lwevent*)(_lw_revents[i].data.ptr);
        if(ev->fd == -1) continue; /* skip invalid events */

        revents = _lw_revents[i].events;
        if(revents & ev->autodestroy) {
            lwevent_free(ev);
            continue;
        }

        ev->callback(ev, revents);
    }

    /* possibly resize returned events array */
    ++_lw_revents_count;
    if(i == ret) ++_lw_revents_overcount;
    if(_lw_revents_overcount >= 4 && _lw_revents_overcount >= (_lw_revents_count >> 2)) {
        new_revents = malloc(sizeof(struct epoll_event) * (_lw_revents_size << 1));
        if(new_revents) {
            free(_lw_revents);
            _lw_revents = new_revents;
            _lw_revents_size <<= 1;
        }
    }

    /* free any invalid struct lwevent objects */
    for(ev = _lwevent_invalid_head; ev; ev = _lwevent_invalid_head) {
        _lwevent_invalid_head = _lwevent_invalid_head->invalid_next;
        free(ev);
    }

    return ret;
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
