/* liblwevent/src/liblwevent/300_fdevent.h
 *
 *  (c)2007, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/*! \defgroup fdevent File descriptor event management

This module contains functions for managing file descriptor-based events. For each file descriptor
to be monitored, a struct lwevent is instantiated with lwevent_new(). This opaque object contains
all the details needed to provide callbacks when events occur (see section \ref fdparams for more
information).

*/
/*!@{*/



/*! \brief Opaque file descriptor event structure.

This structure is associated with a single file descriptor. It holds the details used to
implement callbacks whenever the file descriptor is ready for I/O. The structure cannot be
dereferenced; instead, the functions used in the section \ref fdparams can be used to store or
retrieve associated values.

*/
struct lwevent
#ifdef DOXYGEN
{
    /*! \brief Dummy variable for doxygen */
    int dummy;
}
#endif
;



/*! \brief Event callback function pointer.

\param ev Pointer to event structure.
\param revents Bitmask of returned events (see \c epoll(7)).

This is the type of function pointer used for event callbacks. The event callback function will be
called from lwevent_wait() or lwevent_loop() with a pointer to the associated event structure and
a bitmask of events which have occurred.

*/
typedef void (*lwevent_callback)(struct lwevent* ev, int revents);



/*! \brief Create new event structure associated with a file descriptor.

\param fd File descriptor to monitor.
\param events Bitmask of events to watch for, and epoll flags (see \c epoll(7)).
\param callback Pointer to function to call whenever an event occurs.
\returns Pointer to new lwevent structure on success.
\retval 0 on error (and see \a errno).

This function will create and register with \c epoll(7) a lwevent structure associated with the
file descriptor \a fd. Initially, the object will have the callback function specified in
\a callback, but this (and other attributes) can be changed; see section \ref fdparams.

Once a lwevent structure has been created for \a fd, \a fd should not be passed to \c close(2)
directly. Instead, when it is time to close the file descriptor, pass the associated lwevent
structure to lwevent_free().

\note See lwevent_set_events() for a discussion of valid flags for \a events.

*/
struct lwevent* lwevent_new(int fd, int events, lwevent_callback callback)
    __attribute__((malloc, nonnull, warn_unused_result));



/*! \brief Free an event structure and close its associated file descriptor.

\param ev Event structure. May be 0.

This function will close the file descriptor associated with \a ev, call the destructor if one is
registered (see section \ref fdparams), and free the memory used by \a ev. It may be called at any
time in the event loop. If called with a null argument, it is a no-op.

Whenever this function is called, the object \a ev is marked as invalid. If it has already been
marked as invalid, the function is a no-op. Otherwise, the file descriptor is immediately closed
with \c close(2), and the destructor is also called. The object \a ev itself is not actually freed
at this point; it is placed onto a list of invalid objects, and is only freed at the very end of a
call to lwevent_wait().

This behaviour is what allows an object to be freed at any time without fear of crashing. If two
events are returned from lwevent_wait(), and the callback of the first event calls lwevent_free()
on the second event's object, the second callback can be aborted (since the event object will still
exist, but is marked as invalid). Further objects may call lwevent_free() on the invalid object
without consequence. At the end of the event loop, once no more callbacks are queued, the invalid
objects are freed. This means that you cannot continue to refer to invalid objects after a completed
call to lwevent_wait().

*/
void lwevent_free_and_close(struct lwevent* ev);



/*! \brief Free an event structure.

\param ev Event structure. May be 0.

This function will call the registered destructor of \a ev (see \ref fdparams) and free any memory
used by \a ev. It may be called at any time in the event loop. If called with a null argument, it is
a no-op. The file descriptor associated with \a ev is not modified (but it is deregistered from the
event loop).

*/
void lwevent_free(struct lwevent* ev);



/*! \brief Temporarily deactivate an event structure.

\param ev Event structure.
\retval 0 on success.
\retval -1 on error (and see \a errno).

Sometimes, it is useful to temporarily disable event monitoring for an object, without affecting its
file descriptor. This function provides a method for achieving this. The object can later be
reactivated with lwevent_reactivate(). Deactivated objects may still be operated on and destroyed.

*/
int lwevent_deactivate(struct lwevent* ev) __attribute__((nonnull, warn_unused_result));



/*! \brief Reactivate an event structure after temporary deactivation.

\param ev Deactivated event structure.
\retval 0 on success.
\retval -1 on error (and see \a errno).

Reactivates event monitoring for an object that was temporarily deactivated with 
lwevent_deactivate().

*/
int lwevent_reactivate(struct lwevent* ev) __attribute__((nonnull, warn_unused_result));



/*!@}*/
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
