/* liblwevent/src/liblwevent/300_fdparams.h
 *
 *  (c)2007, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/*! \defgroup fdparams File descriptor object parameters

The functions in this module allow the association of several parameters with each file descriptor
event object (struct lwevent).

*/
/*!@{*/



/*! \brief Event object destructor function pointer.

\param ev Event object being destroyed.

This type is a pointer to a destructor function suitable for cleaning up lwevent structures. The
destructor is called from within lwevent_free(). If no destructor is specified, no action is taken.

*/
typedef void (*lwevent_dtor)(struct lwevent* ev);



/*! \brief Retrieve file descriptor associated with object. */
int lwevent_get_fd(const struct lwevent* ev) __attribute__((nonnull, warn_unused_result));

/*! \brief Retrieve bitmask of events being monitored. */
int lwevent_get_events(const struct lwevent* ev) __attribute__((nonnull, warn_unused_result));

/*! \brief Retrieve pointer to callback function. */
lwevent_callback lwevent_get_callback(const struct lwevent* ev) 
    __attribute__((nonnull, warn_unused_result));

/*! \brief Retrieve user pointer. */
void* lwevent_get_user(const struct lwevent* ev) __attribute__((nonnull, warn_unused_result));

/*! \brief Retrieve pointer to destructor function. */
lwevent_dtor lwevent_get_dtor(const struct lwevent* ev) __attribute__((nonnull, warn_unused_result));

/*! \brief Retrieve autodestroy events bitmask. */
int lwevent_get_autodestroy(const struct lwevent* ev) __attribute__((nonnull, warn_unused_result));



/*! \brief Change monitored events.

\param ev Event object.
\param events New bitmask of events to monitor (see \c epoll_ctl(2)).
\retval 0 on success.
\retval -1 on error (and see \a errno).

Changes the events being monitored for the file descriptor associated with \a ev. This calls 
\c epoll_ctl(2), which may fail; a return value of 0 indicates success.

Only some values are valid for the \a events bitmask (non-authoritative list: \c EPOLLIN, 
\c EPOLLOUT, \c EPOLLRDHUP, \c EPOLLPRI, \c EPOLLET, \c EPOLLONESHOT). Returned events may have 
values not specified in \a events (\c EPOLLERR, \c EPOLLHUP). See the \c epoll_ctl(2) man page for
details.

\note Although \c EPOLLONESHOT is supported by the library, it must be used with care. Whenever an
    object with this flag set returns an event, the object must be reactivated with the
    lwevent_reactivate() function.

*/
int lwevent_set_events(struct lwevent* ev, int events)
    __attribute__((warn_unused_result, nonnull));



/*! \brief Change callback function.

\param ev Event object.
\param callback Pointer to new callback function.

Changes the callback function associated with \a ev.

*/
void lwevent_set_callback(struct lwevent* ev, lwevent_callback callback)
    __attribute__((nonnull));



/*! \brief Set user pointer.

\param ev Event object.
\param user Pointer to new user object. May be 0 to clear.

Changes the user pointer associated with \a ev.

*/
void lwevent_set_user(struct lwevent* ev, void* user) __attribute__((nonnull));



/*! \brief Set destructor function.

\param ev Event object.
\param dtor Pointer to destructor object to call when \a ev is freed. May be 0 to clear.

Changes the destructor function associated with \a ev. The destructor function (if one has been
registered) is called from within lwevent_free(). There is no default destructor. It can be cleared
by passing \a dtor as 0.

*/
void lwevent_set_dtor(struct lwevent* ev, lwevent_dtor dtor) __attribute__((nonnull));



/*! \brief Set autodestroy events bitmask.

\param ev Event object.
\param autodestroy Bitmask of events which destroy object if received.

The autodestroy functionality causes the event object \a ev to be automatically passed to
lwevent_free() if any specified event occurs. You may pass ~0 to autodestroy on any event. See
\c epoll_ctl(2) for a list of valid event flags (non-authoritative list: \c EPOLLIN, \c EPOLLOUT,
\c EPOLLRDHUP, \c EPOLLPRI, \c EPOLLERR, \c EPOLLHUP).

*/
void lwevent_set_autodestroy(struct lwevent* ev, int autodestroy)
    __attribute__((nonnull));



/*!@}*/
/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
