/* liblwevent/src/liblwevent/400_signalfd.h
 *
 *  (c)2007, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/*! \defgroup signalfd Signals through file descriptor

This module provides an interface to \c signalfd(2), a system call which provides an alternative
signal delivery mechanism: certain signals can be read from a file descriptor queue.

\todo Currently, the \c signalfd(2) interface is very new (in fact, the manpage has not yet been
    released and must be pieced together from http://lkml.org/lkml/2007/9/27/88). Furthermore, it
    is not yet implemented in \a glibc. Until that occurs, a function call wrapper for the system
    call is provided in this library. Eventually this will be removed in favour of the authoritative
    \a glibc interface.

*/
/*!@{*/



/*! \brief Signal info structure.

This is the structure that is returned from a \c read(2) of a signalfd.

\todo Eventually, this will be folded into glibc. At that time, this structure definition will have
    to be removed and the code updated to use the new stuff.

*/
struct signalfd_siginfo {
    uint32_t  ssi_signo;       /*!< si_signo */
    int32_t   ssi_err;         /*!< si_errno */
    int32_t   ssi_code;        /*!< si_code */
    uint32_t  ssi_pid;         /*!< si_pid */
    uint32_t  ssi_uid;         /*!< si_uid */
    int32_t   ssi_fd;          /*!< si_fd */
    uint32_t  ssi_tid;         /*!< si_tid */
    uint32_t  ssi_band;        /*!< si_band */
    uint32_t  ssi_overrun;     /*!< si_overrun */
    uint32_t  ssi_trapno;      /*!< si_trapno */
    int32_t   ssi_status;      /*!< si_status */
    int32_t   ssi_svint;       /*!< si_int */
    uint64_t  ssi_svptr;       /*!< si_ptr */
    uint64_t  ssi_utime;       /*!< si_utime */
    uint64_t  ssi_stime;       /*!< si_stime */
    uint64_t  ssi_addr;        /*!< si_addr */
    uint8_t   ssi_pad[48];     /*!< Pad size to 128 bytes (allow space for additional fields in the future) */
};



/*! \brief System call wrapper for \c signalfd(2).

\param fd File descriptor to modify (pass -1 to create a new file descriptor).
\param mask Signal mask. Any signals set in \a mask will be delivered through \a fd (but see below).
\returns File descriptor.
\retval -1 on error (and see \a errno).

System call wrapper function for \c signalfd(2). Any signals turned on in \a mask will be delivered
through the file descriptor \a fd. However, these signals should also be masked from the process
using \c sigprocmask(2), since otherwise the two delivery mechanisms will race.

Passing -1 in \a fd causes a new file descriptor to be allocated. It can be polled, closed and read
normally. It will return struct signalfd_siginfo objects when read. Passing an existing signalfd
file descriptor in \a fd will allow the \a mask to be modified.

\todo Eventually, this system call wrapper function will be removed in favour of \a glibc's
    authoritative version.

*/
int signalfd(int fd, const sigset_t* mask);



/*! \brief Event-driven signal receiver object.

This object implements an event-driven signal receiver based on \c signalfd(2). The object contains
details such as the callback function for each received signal.

*/
struct lwevent_signal
#ifdef DOXYGEN
{
    /*! \brief Dummy variable for doxygen; this structure is really opaque. */
    int dummy;
}
#endif
;



/*! \brief Pointer to signal callback function. */
typedef void (*lwevent_signalcb)(const struct signalfd_siginfo* ssi);



/*! \brief Create and register event-driven signal receiver object.

\param mask Set of signals to catch.
\param callback Callback for received signals.
\returns Pointer to new receiver object.
\retval 0 on error (and see \a errno).

Creates a signal receiver file descriptor using \c signalfd(2), and requests that the signals in
\a mask are received through it. Also calls \c sigprocmask(2) to block the signals in \a mask, and
ensure the delivery mechanisms do not race.

This function will silently ignore attempts to receive \c SIGKILL, \c SIGSTOP, \c SIGBUS, \c SIGFPE,
\c SIGILL and \c SIGSEGV even if they are set in \a mask. Such signals either cannot be redirected
or would cause serious misbehaviour if they were not received immediately through the normal
mechanism.

*/
struct lwevent_signal* lwevent_signalfd(const sigset_t* mask, lwevent_signalcb callback)
    __attribute__((malloc,nonnull,warn_unused_result));



/*! \brief Retrieve signal receiver's signal set. */
void lwevent_signalfd_getsigs(const struct lwevent_signal* si, sigset_t* mask)
    __attribute__((nonnull));

/*! \brief Retrieve signal receiver's callback function. */
lwevent_signalcb lwevent_signalfd_getcb(const struct lwevent_signal* si)
    __attribute__((nonnull));



/*! \brief Modify signal receiver's signal set.

\param si Signal receiver object.
\param mask The new mask to enact.
\retval 0 on success.
\retval -1 on error (and see \a errno).

Changes the set of signals being received by \a si to those set in \a mask. See lwevent_signalfd()
for a discussion of the full behaviour of the receiver.

*/
int lwevent_signalfd_setsigs(struct lwevent_signal* si, const sigset_t* mask)
    __attribute__((nonnull,warn_unused_result));



/*! \brief Modiy signal receiver's callback function. */
void lwevent_signalfd_setcb(struct lwevent_signal* si, lwevent_signalcb callback)
    __attribute__((nonnull));



/*! \brief Close signal receiver.

\param si Signal receiver object. May be 0.

Closes a signal receiver object, unblocking the signals it blocked with \c sigprocmask(2). If \a si
is null, this is a no-op.

*/
void lwevent_signalfd_close(struct lwevent_signal* si);



/*! \brief Set up a default signal receiver.

\returns Pointer to new receiver object.
\retval 0 on error (and see \a errno).

Sets up a default signal receiver object. This object will catch \c SIGINT and \c SIGTERM and set
the lwevent_loop_exit flag to non-zero. Other signals are left for the normal delivery mechanism.

*/
struct lwevent_signal* lwevent_signalfd_default(void) __attribute__((malloc,warn_unused_result));



/*!@}*/


/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4:syntax=c.doxygen
*/
